/* SAMPLE EYELINK EXPT FOR WIN32 PLATFORM  */
/*      (c) 1997 by SR Research Ltd.       */
/*      27 November '97 by Dave Stampe     */
/*       For non-commercial use only       */
/*                                         */
/*   This file can serve as a start        */
/*   for programming most experiments      */

/* Windows 95/NT demonstration experiment (C version) */
/* This file: Gaze-controlled selection, including on-line drift correction */


// #define EA_DEBUG 42 /* *** FOR DEBUGGING ONLY *** */
#define GAZECURSOR 42

#ifdef EA_DEBUG
#ifndef GAZECURSOR
#define GAZECURSOR 42
#endif
#endif


/* *** 3/2001 - was w32_gcontrol_trial way before (and w32_bitmap_trial) *** */
/* *** Modified by Eric Auer: 1000 regions, no longer highlights regions *** */
/* *** but instead does only logging. Gaze at first region activates     *** */
/* *** stimulus presentation (first region is marked by gaze target      *** */
/* *** before) trial ends if valid key is pressed or timeout occurs      *** */


/* This module is intended only for private development of experiments       */
/* Use is granted for non-commercial applications by Eyelink licencees only  */

#include <windows.h>
#include <windowsx.h>

#include "eyelink.h"
#include "w32_exptsppt.h"
#include "w32_demo.h"	/* header file for this experiment */

#include "global_regions.h"     /* *** has rgn[] *** */
#include "reading_loadscript.h" /* *** trial struct! *** */

#include "string.h"		/* snprintf or sprintf or whatever */

int last_region = -1;	  // region with gaze at last update
int current_region = -1;  // region currently accumulating gaze time
long dwell_threshold = 700;	// msec of gaze in region required to trigger
int used_regions = 0; 		/* *** */


/*********** DISPLAY GRAPHICS ************/

// see w32_bitmap_trial.c for draw_grid_to_bitmap() and display_bitmap()

              // inverts RECT area to highlight/unhighlight selection
void invert_rect(RECT *r) 
{                      
  HDC hdc;         // display drawing context

  if(!full_screen_window) return;    // make sure window is still open 
  hdc = GetDC(full_screen_window);   // get drawing context 

  SetROP2(hdc, R2_NOT);                                  // operation = invert
  Rectangle(hdc, r->left, r->top, r->right, r->bottom);  // invert rectangle (don't need brush for invert)
  SetROP2(hdc, R2_COPYPEN);                              // reset GDI mode
  ReleaseDC(full_screen_window, hdc);                    // release context
}


/********* GAZE CONTROL REGIONS *********/

	// This code must determine:
	// - which region an update is in
	// - update region dwell times
	// - trigger on dwell time
	// - do fixup for drift

		// process triggering of a new region:
                // remove highlight from old region, highlight new region
		// *** removed highlighting stuff!
void trigger_region(int region)
{
  int i;
  
  for(i=0; i<used_regions; i++)   // scan thru all regions
    { rgn[i].triggered = 0;
    }
  rgn[region].triggered = 1;
} 			


           // Clear region data when gaze switches to new region	
void reset_regions(void)
{
  int i;
  
  for(i=0; i<used_regions; i++)   // reset data for all regions
    {
      rgn[i].avgxsum = 0;   
      rgn[i].avgysum = 0;
      rgn[i].dwell = 0;
    }
  current_region = -1;      // no region currently selected
  last_region = -1;
} 			

	    // Initial setup of region data
void init_regions(void)
{
  int i;
  int x,y;
  
  for(i=0;i<NREGIONS;i++)   	// For all regions:
    {
      /* *********************************************************** */
      /* *** Change this to use regions matching the text trial! *** */
      /* *** only dummy code for now!                            *** */
      /* *** Even better: combine this with TextExtended2 code   *** */
      /* *** that is: w32_text_bitmap should set up rgn[] !!!    *** */
      /* *********************************************************** */
      x = (i%40) * (SCRWIDTH / 40) + (SCRWIDTH / 80); // center
      y = (i/40) * (SCRHEIGHT / 25) + (SCRHEIGHT / 50); 
      rgn[i].cx = x;		// record center for drift correction
      rgn[i].cy = y;
      SetRect(&(rgn[i].rdraw), x,y,x,y);
      // *** InflateRect does centered resize, seems to be from Win
      InflateRect(&(rgn[i].rdraw), SCRWIDTH/120, SCRHEIGHT/75);
      // defined highlighting rectangle
      SetRect(&(rgn[i].rsense), x,y,x,y);
      InflateRect(&(rgn[i].rsense), SCRWIDTH/80, SCRHEIGHT/50);
      // defined gaze-accumulation rectangle
 
      // *** this is also very dummy:
      rgn[i].rword[0] = '\0';
      // slprintf(rgn[i].rword,"<%3.3d>",i);
		// *** sad - no snprintf dst num what, using sprintf
	    // *** also missing. what the heck?
	  strcpy(rgn[i].rword,"foo");	// *** last resort solution
      rgn[i].rword[99] = '\0';

      used_regions = 1000;
    }
} 			

	// Determine which region gaze is in
	// return 0-24 for a valid region, -1 if not in any region
	// *** 0..used_regions-1 now
int which_region(int x, int y)
{
  int i;
  POINT pt;
  
  pt.x = x;                    // set POINT with gaze location
  pt.y = y;
  for(i=0; i<used_regions; i++)      // scan all regions for gaze position match
    if(PtInRect(&(rgn[i].rsense), pt))
	{
		return i;
	}
  return -1;                   // not in any region
}

		// *** NEW function to present the stimulus !
		// *** if log_synctime is 1, the log message
		// *** that marks zero time for EDFVIEW will be written
UINT32 present_display(int log_synctime, HBITMAP myBitmap) // returns the start time
{
  UINT32 trial_start;
    // DISPLAY OUR IMAGE TO SUBJECT  by copying bitmap to display
    // By waiting for the display refresh before and after drawing, 
    // we will have an accurate idea of thetime the subject saw the stimulus.

  wait_for_video_refresh();	// synchronize before drawing so all seen at once
  display_bitmap(full_screen_window, myBitmap, 0, 0);  
  				// COPY BITMAP to display

  wait_for_video_refresh();  	 // synchronize to refresh so all display sent to monitor
  trial_start = current_time();  // record the display onset time 
  eyemsg_printf("DISPLAY ON");	 // message for RT recording in analysis 
  if (log_synctime != 0)
    eyemsg_printf("SYNCTIME");	 // message marks zero-plot time for EDFVIEW 

  DeleteObject(myBitmap);        // Delete the bitmap 
  return trial_start;
}


		// Process a fixation-update event:
		// Detect and handle a switch between regions
		// Otherwise, accumulate time and position in region
		// Trigger region when time exceeds threshold
void process_fixupdate(int x, int y, long dur,
					   PTRIALPARAMS myTrial, HBITMAP gbm)
{
   int avgx, avgy;
#ifdef GAZECURSOR
   RECT livecursor;			// *** show real time cursor for debugging
#endif
   int i = which_region(x, y);     // which region is gaze in
   avgx = 0;
   avgy = 0;
#ifdef GAZECURSOR
   SetRect(&livecursor, x-2,y-2,x+2,y+2);
   invert_rect(&livecursor);	// *** show real time cursor for debugging
#endif
   
   if(i == -1)		    // NOT IN ANY REGION:
     {		// allow one update outside of a region before resetting
       if(last_region == -1)	// 2 in a row: reset all regions
         {
           if (current_region != -1)
           {
              /* *** if before_last is not -1, log that the gaze *** */
              /* *** just has definitely left the building :-)   *** */
              i = current_region;
              eyemsg_printf("LEFT WORD %d %ld %ld %ld %ld %ld", 
                            i, avgx, avgy, rgn[i].cx, rgn[i].cy, rgn[i].dwell);
#ifdef EA_DEBUG
			// *** tell debugger about result
			get_new_font("Times Roman", 24, 1);
			wait_for_video_refresh();
			graphic_printf(target_foreground_color, -1, 1, SCRWIDTH/2, 24,
				"left region %ld",i);
			msec_delay(25);
			wait_for_video_refresh();
			graphic_printf(target_background_color, -1, 1, SCRWIDTH/2, 24,
				"left region %ld",i);
#endif
           }
           reset_regions();	// *** 2 fixations in "empty space" in a row
           			// *** info: this clears current_region etc.!
         }
     }
   else if(i == current_region)	   // STILL IN CURRENT REGION 
     {
       rgn[i].dwell += dur;	      // accumulate time, position
       rgn[i].avgxsum += dur * x;
       rgn[i].avgysum += dur * y;
       if(rgn[i].dwell>dwell_threshold && !rgn[i].triggered)  // did this region trigger yet?
         {
           trigger_region(i);		        // TRIGGERED:
           avgx = rgn[i].avgxsum / rgn[i].dwell;    // compute avg. gaze position
           avgy = rgn[i].avgysum / rgn[i].dwell;
                  	// correct for drift (may cause false saccade in data)
			if ((i == 0)	&& (myTrial->bGazeActivate == TRUE))
	   		// *** only 1st region is suited for drift correction
				{
#ifdef EA_DEBUG
					// *** warn debugger of drift correction
					get_new_font("Times Roman", 24, 1);
					wait_for_video_refresh();
					graphic_printf(target_foreground_color, -1, 1, SCRWIDTH/2, 24,
						"auto-dc: region %d, delta (%ld, %ld)",i,
						(long)rgn[i].cx-avgx, (long)rgn[i].cy-avgy);
					msec_delay(200);
					wait_for_video_refresh();
					graphic_printf(target_background_color, -1, 1, SCRWIDTH/2, 24,
						"auto-dc: region %d, delta (%ld, %ld)",i,
						(long)rgn[i].cx-avgx, (long)rgn[i].cy-avgy);
#endif
					eyecmd_printf("drift_correction %ld %ld %ld %ld",
                            (long)rgn[i].cx-avgx, (long)rgn[i].cy-avgy,
                            (long)rgn[i].cx, (long)rgn[i].cy);
	     /* ************************************************************ */
	     /* *** This is the time when the actual stimulus should be  *** */
	     /* *** made visible and the simplified (1 target only, e.g. *** */
	     /* *** a cross) is removed. Notice the logging style...     *** */
	     /* ************************************************************ */
					eyemsg_printf("TRIGGER MAIN %d %ld %ld %ld %ld %ld", 
                           i, avgx, avgy, rgn[i].cx, rgn[i].cy, rgn[i].dwell);
					i = current_region = -1;	// *** will trigger reset_regions next

					/* *** swap stimuli here - NEW function! *** */
					present_display(0,gbm);	 /* *** show image *** */
				    /* *** 0 means do not log SYNCTIME *** */
				}
			// *** next is generic trigger stuff
	  		// Log triggering to EDF file
	  		// *** Hint: Log the word associated to the number, too
			if (i >= 0)
				{
					eyemsg_printf("TRIGGER WORD %d %ld %ld %ld %ld %ld", 
                             i, avgx, avgy, rgn[i].cx, rgn[i].cy, rgn[i].dwell);
					// *** actually not really useful for most experiments
				}
         } 
     }
   else if(i == last_region)	// TWO UPDATES OUTSIDE OF CURRENT REGION:
   				// SWITCH TO NEW REGION
     {
       reset_regions();              // clear and initialize accumulators
       rgn[i].dwell = dur;	      
       rgn[i].avgxsum = dur * x;
       rgn[i].avgysum = dur * y;
       current_region = i;           // now working with new region
       /* *** log that the gaze went to a new word! *** */
#ifdef EA_DEBUG
			// *** tell debugger about result
			get_new_font("Times Roman", 24, 1);
			wait_for_video_refresh();
			graphic_printf(target_foreground_color, -1, 1, SCRWIDTH/2, 24,
				"entered region %ld",i);
			msec_delay(25);
			wait_for_video_refresh();
			graphic_printf(target_background_color, -1, 1, SCRWIDTH/2, 24,
				"entered region %ld",i);
#endif
       eyemsg_printf("ENTERED WORD %d %ld %ld %ld %ld %ld %s", 
                     i, avgx, avgy, rgn[i].cx, rgn[i].cy, rgn[i].dwell,
                     rgn[i].rword); // *** only place where rword is used
       
     }
  last_region = i;

#ifdef GAZECURSOR
  wait_for_video_refresh();  
  msec_delay(20);
  SetRect(&livecursor, x-2,y-2,x+2,y+2);
  wait_for_video_refresh();
  invert_rect(&livecursor);	// *** hide real time cursor again
#endif
}


/********* RUN A GAZE-CONTROL TRIAL  *******/

		// End recording: adds 100 msec of data to catch final events
                // Reset configuration changes made previously
static void end_trial(void)
{
  clear_full_screen_window(target_background_color);    // hide display 
  msec_delay(100);       // record additional 100 msec of data 
  stop_recording();
			 // Reset link data, disable fixation event data
  eyecmd_printf("link_event_filter = LEFT,RIGHT,FIXATION");
  eyecmd_printf("fixation_update_interval = 0");
  eyecmd_printf("fixation_update_accumulate = 0");
}

	/* *** 3/2001: heavy changes by Eric Auer - not THAT well documented *** */

	/* Run a gaze-control trial, recording to EDF file and sending FIXUPDATE events through link */
	/* This example draws to a bitmap, then copies it to display for fast stimulus onset */ 

	// The order of operations is:
	// - Set trial title, ID for analysis
        // - Draw to bitmap and create EyeLink display graphics
	// - Create regions
	// - Drift correction
	// - start recording
	// - start recording
	// - loop till button press, timeout, or abort, and process FIXUPDATE events	
	// - stop recording, handle abort and exit

        // This trial draws its own bitmap and creates control regions	 

int reading_trial(HBITMAP gbm, PTRIALPARAMS myTrial) 
			// *** new name, new functionality!
{
  UINT32 trial_start;	// trial start time (for timeout) 
  UINT32 time_limit;	// *** from myTrial here...
  int button;		// the button pressed (0 if timeout) 
  int error;            // trial result code
  int key;              // key pressed

  ALLF_DATA evt;        // buffer to hold sample and event data
  int eye_used = -1;    // indicates which eye's data to display
  int i,j;		// *** added j

// ***  HBITMAP gbm;          // The bitmap containing the stimulus display

   last_region = -1;
   current_region = -1;

  time_limit = myTrial->lTimeLimit;	// *** has to be there later
  used_regions = rgn[NREGIONS-1].dwell;
  if ((used_regions<1) || (used_regions>NREGIONS))
  { used_regions = NREGIONS;
  }		// *** speed up things based on evil hack in w32_text_bitmap
		// *** but at least we check for sanity...

#ifdef EA_DEBUG
//  i = present_display(0,gbm); // *** reveal everything to the debugger :-)
  for (i=0; i<used_regions; i++)
  {
	  invert_rect(&rgn[i].rdraw); // *** reveal regions to the debugger
	  invert_rect(&rgn[i].rsense); // *** reveal regions to the debugger
  }
#endif

	   // This supplies the title at the bottom of the eyetracker display 
// *** commenting this out - reading_trials.h has already done it
// ***  eyecmd_printf("record_status_message 'GAZE CONTROL TRIAL' ");

	   // Always send a TRIALID message before starting to record. 
	   // It marks the start of the trial and should precede any other messages
	   // It should contain trial condition data required for analysis.
// *** same here!
// ***  eyemsg_printf("TRIALID GAZECTRL");

  	// Before recording, we place reference graphics on the EyeLink display
  set_offline_mode();		 // Must be offline to draw to EyeLink screen
// *** strange but true: this is done in reading_trials, too!
// ***  gbm = draw_grid_to_bitmap();   // Draw bitmap and EyeLink reference graphics

	   // DO PRE-TRIAL DRIFT CORRECTION     *** IF NOT GAZE ACTIVATED
	   // We repeat if ESC key pressed to do setup. 

  while(myTrial->bGazeActivate == FALSE) // *** Gaze Activation replaces this!
    {              // Check link often so we can exit if tracker stopped
      if(!eyelink_is_connected()) return ABORT_EXPT;
	   // We let do_drift_correct() draw target in this example
	   // 3rd argument would be 0 if we already drew the display
      error = do_drift_correct((INT16)(SCRWIDTH/2), (INT16)(SCRHEIGHT/2), 1, 1);
           // repeat if ESC was pressed to access Setup menu 
      if(error!=27) break;
        // The screen was erased after drift correction because we had it
        // draw the target. If we drew the target, we could erase it now.
    }
  
        // Configure EyeLink to send fixation updates every 50 msec
  eyecmd_printf("link_event_filter = LEFT,RIGHT,FIXUPDATE");
  eyecmd_printf("fixation_update_interval = 50");
  eyecmd_printf("fixation_update_accumulate = 50");
 	
// *** this is done in text_bitmap, called from reading_trials, before
// *** text_bitmap writes to rgn[] directly for that (how un-stylish!)
// *** *** init_regions();	// Initialize regions for this display
  
	// Start data recording to EDF file, BEFORE DISPLAYING STIMULUS 
	// You should always start recording 50-100 msec before required
	// otherwise you may lose a few msec of data 
// *** w32_exptsppt.h says: file{samples,events}, link{samples,events}
// *** added user control over EDF stuff, kind of

  error = start_recording( /* uses INT16 values */
	  /* *** DONT ASK ME WHY BUT SAMPLES HAVE TO BE ON FOR EDFVIEWX *** */
// #ifdef EA_DEBUG
			1, /* *** write samples to file for debugging *** */
// #else
//			0, /* *** no samples at all today! *** */
// #endif
			(INT16)((myTrial->bLogStream == TRUE) ? 1 : 0), /* BOOL is int */

			0, /* send no samples through link */
			1);	/* send events (like gazes) through link, not samples */
  
  if(error != 0)           // ERROR: couldn't start recording
    {
      DeleteObject(gbm);   // Be sure to delete bitmap before exiting!
      return error;        // Return the error code
    }
  msec_delay(100);	   // record for 100 msec before displaying stimulus 

	// *** if we have gaze activation, delay this!
  if (myTrial->bGazeActivate == FALSE)
  {
     trial_start = present_display(1,gbm);	/* *** show image *** */
     					/* *** 1 means log SYNCTIME    *** */
	// get_new_font("Times Roman", 24, 1);
	// Print a title for the trial (for demo only)
	// graphic_printf(target_foreground_color, -1, 1, SCRWIDTH/2, 24,
	//                "Gaze Control Trial");
  } // *** end of normal display setup
  else
  {
    wait_for_video_refresh();
#ifndef EA_DEBUG
	// *** dont hide hints for debugger!
    clear_full_screen_window(target_background_color);    // hide display 
#endif
    invert_rect(&rgn[0].rdraw);	// *** gaze target: invert the
    				// *** region where the 1st word will be!
    wait_for_video_refresh();
    // *** hope it is ok to take the subject starting to look at the point
    // *** where the text will start as part of the trial?
    trial_start = current_time();  // record the display onset time 
    eyemsg_printf("TARGET ON");	 // message for RT recording in analysis 
    eyemsg_printf("SYNCTIME");	 // message marks zero-plot time for EDFVIEW 
    
  }
  	
     // Now get ready for trial loop
  eyelink_flush_keybuttons(0);   // reset keys and buttons from tracker 
  while(getkey()) {};		 // dump any pending local keys 


     // Trial loop: till timeout or response -- added code for processing FIXUPDATE events
  while(1) 
    {                            // First, check if recording aborted 
      if((error=check_recording())!=0) return error;  

                                 // Check if trial time limit expired
      if(current_time() > trial_start+time_limit)
        {
          eyemsg_printf("TIMEOUT");    // message to log the timeout 
	  end_trial();                 // local function to stop recording
	  button = 0;                  // trial result message is 0 if timeout 
	  break;                       // exit trial loop
	}

                // Check for key press, and also allows Windows to pass messages
                // getkey() returns ESC to stop trial, or TERMINATE_KEY to end experiment
      key = getkey();         // get key or termination code
      button = 0;

      if(key==TERMINATE_KEY)  // ALT-F4 or CTRL-C
	{
	   end_trial();         // local function to stop recording
	   return ABORT_EXPT;   // return this code to terminate experiment
	 }
       else if(key==27)       // ESC key
         {
	   end_trial();         // local function to stop recording
	   return SKIP_TRIAL;   // return this code if trial terminated
         }
       else if ((myTrial->lUsedButtons)>0)	// *** scan iButtonList here!
         {
           for (j=0; j<(myTrial->lUsedButtons); j++)
             if (myTrial->iButtonList[j] == key)
               {
                 // *** using the index in iButtonList for logging --
                 // *** but we add the key code in the ANSWERBUTTON log event
                 button = j | 128;  // *** not elegant at all but should work
               }
         }

       if (button != 0)	// *** used to check if a key on the list was pressed
         {
           eyemsg_printf("ANSWERBUTTON %d KEY %d", button, key);
           end_trial();	// local function to stop recording
           break;	// exit trial loop
         }

		/* BUTTON RESPONSE TEST */
		// Check for eye-tracker buttons pressed
		// This is the preferred way to get response data or end trials	
      button = eyelink_last_button_press(NULL);
      if(button!=0)       // button number, or 0 if none pressed
	{
	   eyemsg_printf("ENDBUTTON %d", button);  // message to log the button press
	   end_trial();                            // local function to stop recording
	   break;                                  // exit trial loop
	 }
  
		// GET FIXUPDATE EVENTS, PROCESS

      i = eyelink_get_next_data(NULL);    // Check for data from link
      if(i == FIXUPDATE)	  // only process FIXUPDATE events
	{
	   if(eye_used==-1)       // If we don't yet know which eye to process, set it
	     {
	        eye_used = eyelink_eye_available();  // find which eye's data present
	        switch(eye_used)			
                  {			       // select eye, add annotation
	            case RIGHT_EYE:
	              eyemsg_printf("EYE_USED 1 RIGHT");
	              break;
	            case BINOCULAR:
	              eye_used = LEFT_EYE;
	            case LEFT_EYE:
	              eyemsg_printf("EYE_USED 0 LEFT");
	              break;
	          }
	     }
	   eyelink_get_float_data(&evt);   // get a copy of the FIXUPDATE event 
	   if(evt.fe.eye == eye_used)	   // only process if it's from the desired eye?
	     {                             // get average position and duration of the update 
	       process_fixupdate((int)evt.fe.gavx, (int)evt.fe.gavy,     // Process event 
	       			 evt.fe.entime-evt.fe.sttime, myTrial, gbm);
			// giving it myTrial and gbm is new
	     }
	}
    }       // end of loop
	   // report response result: 0=timeout, else button number
	   // *** button number will be 128 or greater for keyboard answers
	   // *** matching the allowed answer list from the control script
  eyemsg_printf("TRIAL_RESULT %d", button);
	   // Call this at the end of the trial, to handle special conditions
  return check_record_exit();
}

