/* SAMPLE EYELINK EXPT FOR WIN32 PLATFORM  */
/*      (c) 1997 by SR Research Ltd.       */
/*      27 November '97 by Dave Stampe     */
/*       For non-commercial use only       */
/*                                         */
/*   This file can serve as a start        */
/*   for programming most experiments      */

/* Windows 95/NT demonstration experiment (C version) */
/* Sequence block of trials, including calibration and error handling */
/* Something similar can be used for each block of an experiment      */

/* This module is intended only for private development of experiments       */
/* Use is granted for non-commercial applications by Eyelink licencees only  */

/* ***************************************************** */
/* *** This was w32_text_trials.c from TextExtended2 *** */
/* *** Modified 3/2001 by Eric Auer                  *** */
/* ***************************************************** */


// #define EA_DEBUG 42		/* *** ONLY FOR DEBUGGING *** */


#include <windows.h>
#include <windowsx.h>

#include "eyelink.h"
#include "w32_exptsppt.h"
#include "w32_demo.h"  /* header file for this experiment */

#include "global_regions.h" /* *** added this (Eric)  *** */
#include "reading_loadscript.h" /* *** ... *** */

#include "StoringAnImage.h"
#include "LoadingATextFile.h"
#include "IniFile.h"
#include "LogTable.h"
#include "AddUnicode.h"


/*********** TRIAL SETUP AND RUN **********/

int gnTrials = 0;

         // FOR EACH TRIAL:
         // - set title, TRIALID
         // - Create bitmaps and EyeLink display graphics
         // - Check for errors in creating bitmaps
         // - Run the trial recording loop
         // - Delete bitmaps
         // - Return any error code

              // Given trial number, execute trials
              // Returns trial result code

// and store Bitmap to File
int do_text_trial(int num)
{
  HBITMAP bitmap;
  RECT margins;  // margins for text
  int i;
  TCHAR pszFilename[256];
//  TCHAR pszNum[12];
  TCHAR* pszText;
  PTRIALPARAMS myTrial;

  BOOL bRet; // ***
  int trialPages = 1; // ***

  myTrial = GetTrial(num);	// *** a structure now!
  if (myTrial == NULL) return ABORT_EXPT;

//  itoa(num, pszNum, 10);
//  lstrcpy(pszFilename, TEXT("trial")); // ***
//  lstrcat(pszFilename, pszNum);
//  lstrcat(pszFilename, TEXT("page1.bmp")); // *** only 1pg/trial for now
  lstrcpy(pszFilename, myTrial->pszTrialIdent); // *** only 1pg/trial for now
  lstrcat(pszFilename, TEXT("-1.bmp"));
   
  margins.top = SCRHEIGHT/20;
  margins.left = SCRWIDTH/20;
  margins.right = SCRWIDTH - SCRWIDTH/20;
  margins.bottom = SCRHEIGHT - SCRHEIGHT/20;

         // This supplies the title at the bottom of the eyetracker display 
  eyecmd_printf("record_status_message 'READING TRIAL %d/%d PAGE 1/1 IDENT %s' ",
                num, gnTrials, myTrial->pszTrialIdent); // ***

        // Always send a TRIALID message before starting to record. 
        // It should contain trial condition data required for analysis.
  eyemsg_printf("TRIALID %sPAGE1", myTrial->pszTrialIdent);
  	// *** limited length??? -> use %d num, not ident

  get_new_font(gIniPara.pszFontName, gIniPara.lFontHeight, 0);
  // get_new_font("Times Roman", SCRHEIGHT/25, 0);
	
  // *** yeah, I know, file access during experiment is not optimal
  bRet = LoadText(myTrial->pszTextFilename, &trialPages); // ***
  if (!bRet)
    {
      eyemsg_printf("TRIAL ERROR");
      MessageBox(full_screen_window, TEXT("Text file not found."),
                 TEXT("Error"), MB_OK | MB_ICONERROR);

	  // *** hope this works:
#ifdef EA_DEBUG
			// *** tell which file is missing
			get_new_font("Times Roman", 24, 1);
			graphic_printf(target_foreground_color, -1, 1, SCRWIDTH/2, 24,
				"Missing: Trial %d [%s]",
				num,
				myTrial->pszTextFilename);
#endif

      return SKIP_TRIAL;
    }

  // ***
  // *** if trialPages > 1, the text file contains not yet usable data!
  // ***
 
  pszText = GetPage(1);
  if (pszText)
  {
  		// *** change text_bitmap to write to rgn[] !!!
	    // *** was RGB(0,0,0), RGB(255,255,255)
	  bitmap = text_bitmap(pszText, READING_TEXT_COLOR, READING_TEXT_BACK,
	                       margins, gIniPara.lLineSpacing, 1);
	  StoreBitmapToFile(full_screen_window, bitmap, pszFilename);

	  i = reading_trial(bitmap, myTrial); // *** not only time limit passed
	  	// *** used info: bGazeActivate, bLogStream, lTimeLimit,
	  	// *** lUsedButtons, int iButtonList[]
 
	  DeleteObject(bitmap);
  }

  ReleaseText();

  return i;
}


/*********** TRIAL LOOP **************/

	/* This code sequences trials within a block */
	/* It calls run_trial() to execute a trial, */
	/* then interperts result code. */
	/* It places a result message in the EDF file */
	/* This example allows trials to be repeated */
	/* from the tracker ABORT menu. */
int run_trials(void)
{
	int i;
	int trial;
	BOOL bRet;

	target_foreground_color = READING_TARGET_COLOR; // set background for calibration
	target_background_color = READING_TARGET_BACK;  // This should match the display 
	set_calibration_colors(target_foreground_color, target_background_color); 
    
  		/* PERFORM CAMERA SETUP, CALIBRATION */
#ifdef EA_DEBUG
	MessageBox(full_screen_window, TEXT("Starting tracker setup now."),
			TEXT("Notice"),
			MB_OK | MB_ICONWARNING); // ERROR WARNING
#endif
	do_tracker_setup();

	// get values from ini file

	if (!ReadParams(&gIniPara))
	{
		MessageBox(full_screen_window, TEXT("No ini file, using defaults."),
			TEXT("Warning"),
			MB_OK | MB_ICONWARNING);
		GetDefaultValues(&gIniPara);
		WriteParams(&gIniPara);
	}

	bRet = LoadScript(gIniPara.pszTextFilename, &gnTrials);
	if (!bRet)
	{
		eyemsg_printf("TRIAL ERROR");
		MessageBox(full_screen_window, TEXT("Script file not found."), TEXT("Error"),
			MB_OK | MB_ICONERROR);
		return 0;
	}
				/* loop through trials */
	for(trial=1;trial<gnTrials;trial++) // *** 1based index!
										// *** but still using <, not <= ...
	{
#ifdef EA_DEBUG
		// *** show when next iteration of trial loop begins ***
		MessageBox(full_screen_window, TEXT("Prepare for next trial."),
			TEXT("Notice"),
			MB_OK | MB_ICONWARNING); // ERROR WARNING
#endif
		if(eyelink_is_connected()==0 || break_pressed())    /* drop out if link closed */
		{
			return ABORT_EXPT;
		}
				/* RUN THE TRIAL */
		LogTableSetPage(trial);
		i = do_text_trial(trial); // *** see above - calls reading_trial

		switch(i)         	/* REPORT ANY ERRORS */
		{
			case ABORT_EXPT:        /* handle experiment abort or disconnect */
				eyemsg_printf("EXPERIMENT ABORTED");
				ReleaseScript();
				return ABORT_EXPT;
			case REPEAT_TRIAL:	  /* trial restart requested */
				eyemsg_printf("TRIAL REPEATED");
				trial--;
				break;
			case SKIP_TRIAL:	  /* skip trial */
				eyemsg_printf("TRIAL ABORTED");
				break;
			case TRIAL_OK:          // successful trial
				eyemsg_printf("TRIAL OK");
				break;
			default:                // other error code
				eyemsg_printf("TRIAL ERROR");
				break;
		}
	}  // END OF TRIAL LOOP

	ReleaseScript();
	return 0;
}

