/* SAMPLE EYELINK EXPT FOR WIN32 PLATFORM  */
/*      (c) 1997 by SR Research Ltd.       */
/*      27 November '97 by Dave Stampe     */
/*       For non-commercial use only       */
/*                                         */
/*   This file can serve as a start        */
/*   for programming most experiments      */

/* Windows 95/NT demonstration experiment (C version) */
/* This file: Copy bitmaps to display, create blank bitmap */

/* This module is intended only for private development of experiments       */
/* Use is granted for non-commercial applications by Eyelink licencees only  */

#include <windows.h>
#include <windowsx.h>

#include "eyelink.h"
#include "w32_exptsppt.h"
#include "w32_demo.h"       /* header file for this experiment */


/************ DISPLAY ENTIRE BITMAP **************/

         // Copy a DDB bitmap to the display 
         // Top-left placed at (x,y)
         // Speed depends on video mode and graphics card
void display_bitmap(HWND hwnd, HBITMAP hbm, int x, int y)
{
  BITMAP bm;        // Bitmap data structure
  HDC hdc;          // Display context (to create memory context)
  HDC mdc;          // Memory context to draw bitmap in
  HBITMAP obm;      // old GDI bitmap

  hdc = GetDC(hwnd);                     // Display DC 
  mdc = CreateCompatibleDC(hdc);         // Memory DC
  obm = SelectObject(mdc, hbm);          // Select bitmap into memory DC
  
  GetObject(hbm, sizeof(BITMAP), &bm);   // Get data on bitmap
                                         // Use BitBlt to copy to display 
  BitBlt(hdc, x, y, bm.bmWidth, bm.bmHeight, mdc, 0, 0, SRCCOPY);
  
  SelectBitmap(mdc, obm);                // Deselect bitmap
  DeleteDC(mdc);                         // get rid of memory DC
  ReleaseDC(hwnd, hdc);
}


/************* DISPLAY PART OF BITMAP **********/

        // Copies a rectangular area of a bitmap to the display
        // Copies rectangle <xs1, ys1, xs2, ys2> of DDB <hbm> 
        // Places top-left corner at <xd, yd> 
        // for maximum drawing speed, xd=xs1 and yd=ys1 (aligned copy)
        // <hbm> is source bitmap, <hwnd> is the window to draw to
void display_rect_bitmap(HWND hwnd, HBITMAP hbm, int xd, int yd, 
                                                 int xs1, int ys1, int xs2, int ys2)
{
  HDC hdc;
  HDC mdc;
  HBITMAP obm;
  int h, w;

  if(!hwnd || !hbm) return;
  w = xs2 - xs1 + 1;
  h = ys2 - ys1 + 1;

  hdc = GetDC(hwnd);                // display drawing context
  mdc = CreateCompatibleDC(hdc);    // memory context
  obm = SelectObject(mdc, hbm);     // select bitmap
  
  BitBlt(hdc, xd, yd, w, h, mdc, xs1, ys1, SRCCOPY);   // copy relevant part to display

  SelectBitmap(mdc, obm);           // release GDI resources
  DeleteDC(mdc);
  ReleaseDC(hwnd, hdc);
}


/************ CREATE BLANK BITMAP ***********/

      // Creates a blank, full-screen sized bitmap, cleared to <bgcolor>
HBITMAP blank_bitmap(COLORREF bgcolor)
{
  HDC hdc;
  HBITMAP hbm;
  HDC mdc;
  HBRUSH oBrush;
  HBITMAP obm;

  hdc = GetDC(NULL);                 
  mdc = CreateCompatibleDC(hdc);     // create display-compatible memory context
  hbm = CreateCompatibleBitmap(hdc, SCRWIDTH, SCRHEIGHT);
  obm = SelectObject(mdc, hbm);      // create DDB bitmap, select into context
                                     
  oBrush = SelectObject(mdc, CreateSolidBrush(bgcolor | 0x02000000L));  // brush to fill with 
  PatBlt(mdc, 0, 0, SCRWIDTH, SCRHEIGHT, PATCOPY);
  DeleteObject(SelectObject(mdc, oBrush));

  SelectBitmap(mdc, obm);     // Release the GDI resources
  DeleteDC(mdc);
  ReleaseDC(NULL, hdc);
  return hbm;                 // Return the new bitmap
}

