/* SAMPLE EYELINK EXPT FOR WIN32 PLATFORM  */
/*      (c) 1997 by SR Research Ltd.       */
/*      27 November '97 by Dave Stampe     */
/*       For non-commercial use only       */
/*                                         */
/*   This file can serve as a start        */
/*   for programming most experiments      */

/* Windows 95/NT demonstration experiment (C version) */
/* Draw left-justified text in margins, to display or bitmap */

/* This module is intended only for private development of experiments       */
/* Use is granted for non-commercial applications by Eyelink licencees only  */

/* *** Changed 3/2001 by Eric Auer to write to rgn[] and added \_  *** */
/* *** \_ is an invisible non breakable space, rgn[] has geom info *** */

#include <windows.h>
#include <windowsx.h>
#include <stdio.h>

#include "eyelink.h"
#include "w32_exptsppt.h"
#include "w32_demo.h"

#include "LogTable.h"
#include "AddUnicode.h"
#include "global_regions.h" // *** added this...


/********** DRAW FORMATTED TEXT ***********/

       // Draw text within margins, left-justified, with word-wrap.
       // Draw to <hdc> context, so it supports display and bitmaps   
       // (bitmap must already be created and selected into context).
       // Draw boxes for each word on EyeLink tracker display if <dotrack> set
       // Use font selected with get_new_font(), and draws it in <color>
       // RECT <margins> sets margins, and <lspace> sets pixels between lines
void draw_text_box(HDC hdc, TCHAR *txt, COLORREF color, RECT margins, int lspace, int dotrack)
{
  LOGTABLEPARA para;
  short x, y;              // drawing position
  TCHAR *sword, *eword;     // word start, end pointer
  int swx, ewx;            // word start, end position
  TCHAR *sispace, *c;       // text pointers
  int height;              // height of characters
  SIZE size;
  TEXTMETRIC tm;
  int i; 		// *** index for rgn[]
  int used_regions;

  if(dotrack) eyecmd_printf("set_idle_mode");   // must be in idle mode to draw graphics
  if(dotrack) eyecmd_printf("clear_screen 0");	// clear eye tracker display

  SetBkMode(hdc, TRANSPARENT);        // set text color
  SetTextColor(hdc, color);
  SelectObject(hdc, current_font);    // Select cached font into context

  GetTextMetrics(hdc, &tm);           // Get height of characters
  height = tm.tmHeight;
  SetTextJustification(hdc, 0, 0) ;   // set normal text spacing
  SetTextAlign(hdc, TA_LEFT | TA_TOP);

  x = (short)margins.left;                   // initial drawing position
  y = (short)margins.top;  
  c = txt;                            // initial text to scan
  para.nLine = 1;
  para.nWordNo = 1;

  /* *************************************************************** */
  /* *** ADD THE \_ NON-BREAKING INVISIBLE SPACE FEATURE HERE    *** */
  /* *************************************************************** */

  // *** rgn[NREGIONS] has int: triggered, cx, cy (the center for d.corr.),
  // *** and long: avg{x,y}sum, dwell (time),
  // *** and RECT: rsense (for gaze), rdraw (for gaze ack),
  // *** and char rword[100] (for the log) -- *** could be unicode TCHAR...
  // *** only 0..used_regions are actually used, update this, too!

  /* *** added writing to rgn[] here -- global_regions.h for that *** */

  used_regions = 0;	// *** returned for reading_trials
  
  while(*c)   // loop through lines of text
    {
nextword:                 // START NEW WORD:
      sispace = c;           
				// scan over any leading spaces
      while(isspace(*c))
	  {
		  c++;
	  }

      if(c!=sispace)          // advance over the spaces (if any)
        {
		  // computes the width and height of the space
          GetTextExtentPoint32(hdc, sispace, c-sispace, &size);                    
          x += (short)size.cx;
        }

      if(*c==0) break;    // handle end-of-text 

	// *** this is the standard "\\n" user-newline feature provided
	// *** by w32_text_bitmap:
      if((*c == '\\') && (*(c+1) == 'n'))        // handle end-of-line *before* word

//    if(*c=='\n')        // handle end-of-line before word
        {
          c+=2;            // skip over end-of-line characters
//        c++;            // skip over end-of-line character

nextline:                        // START NEW LINE:
          x = (short)margins.left;                      // return to start of line
          y += lspace;                           // top of new  line
		  para.nLine++;                          // increments line counter
		  para.nWordNo = 1;                      // word number begins with 1
          if(y+height > margins.bottom)
		  {
			MessageBox(full_screen_window, TEXT("draw_text_box: too much text."),
				TEXT("Warning"),
				MB_OK | MB_ICONERROR); // ERROR WARNING
			break;  // stop if it would go outside margins
		  }
          goto nextword;                         // continue to print
        }
		else	/* *** have to handle ALL backslash sequences or will loop endless! *** */
		{
			if ((*c == '\\') && (*(c+1) != 'n'))
			{
				c+=2;	// *** skip over at least the backslash itself
				/* *** FOR \_ HANDLING: *SKIP* SEQUENCE SO WE STILL    *** */
				/* *** HAVE ONLY ONE WORD BUT INSERT CODE TO SET UP AN *** */
				/* *** ADDITIONAL ENTRY IN THE rgn[] LIST RIGHT HERE ! *** */
			}
		}
      swx = x;            // continue in current line: 

      sword = c;                                  // record start of this word
	  sscanf(c, TEXT("%s"), para.pszWord);              // reads the word
      while(*c && 
		  !isspace(*c) && 
		  *c!='\n' && *c!='\r' && *c!='\\' /* *** backslash always ends a word *** */
		  ) c++;  // scan till end of word
	  if (*c=='\\')
	  {
		/* *** MAYBE DO SOMETHING WITH \_ (non breaking space) HERE *** */
		/* *** BUT CAREFUL, SHOULD BETTER BE HANDLED ABOVE  *** */
	  }
      if(sword==c) goto nextword;                 // no word: continue by skipping spaces

      eword = c;                                  // record end of word
      GetTextExtentPoint32(hdc, sword, eword-sword, &size);   // width of word                 
      ewx = size.cx + swx;                        // position of end of word

      if(ewx > margins.right)    // would it extend past right margin?
        {
          c = sword;                   // restore text pointer
          goto nextline;               // start a new line with current word
        }

      TextOut(hdc, x, y, sword, eword-sword);      // Print the word (to the bitmap)

      if(dotrack)
	  {		// draw box on EyeLink display
		  para.rectWord.left = swx;
		  para.rectWord.top = y;
		  para.rectWord.right = ewx;
		  para.rectWord.bottom = y+height;
          eyecmd_printf("draw_box %d %d %d %d 15", 
			  para.rectWord.left, 
			  para.rectWord.top, 
			  para.rectWord.right, 
			  para.rectWord.bottom); 
//          eyecmd_printf("draw_box %d %d %d %d 15", swx, y, ewx, y+height);
		  LogTable(&para);
		  para.nWordNo++;
	  }

      if(1==1)	// *** write to rgn[] -- always on for now!
          {
          	i = used_regions;
          	// never fail:
          	if (i >= NREGIONS) { used_regions--; i--; };
          	used_regions++;
          	rgn[i].triggered = 0;
          	rgn[i].dwell = 0;
          	rgn[i].avgxsum = 0;
          	rgn[i].avgysum = 0;
          	rgn[i].cx = (swx + ewx) / 2;
          	rgn[i].cy = y + (height / 2);
          	SetRect(&(rgn[i].rdraw), swx, y, ewx, y+height);

			if ((height < 16) || ((ewx-swx) < 16))
			{
          	SetRect(&(rgn[i].rsense), swx+((ewx-swx)/4), y+(   height /4),
          	                          ewx-((ewx-swx)/4), y+((3*height)/4));
			}
			else
			{
          	SetRect(&(rgn[i].rsense), swx+3, y+3,
          	                          ewx-4, y+height-4);
			}

          	for (i=0; i<(eword-sword); i++)
          	  {
          	    // *** this may involve unicode -> text conversion if TCHAR is used
          	    rgn[used_regions-1].rword[i] = sword[i];
          	  }
          	rgn[used_regions-1].rword[eword-sword] = '\0';
          }
      x = ewx;                                     // update current printing location
    }

  SelectObject(hdc, GetStockObject(SYSTEM_FONT));  // deselect the font

  rgn[NREGIONS-1].dwell = used_regions; // *** evil hack to speed up things

}


/************ CREATE BITMAP WITH FORMATTED TEXT ***********/

      // Draw text to full-screen sized bitmap, cleared to <bgcolor>
      // Draw boxes for each word on EyeLink tracker display if <dotrack> set
      // Use font selected with get_new_font(), and draws it in <fgcolor>
      // RECT <margins> sets margins, and <lspace> sets pixels between lines
HBITMAP text_bitmap(TCHAR *txt, COLORREF fgcolor, COLORREF bgcolor, 
                               RECT margins, int lspace, int dotrack)
{
  HDC hdc;
  HBITMAP hbm;
  HDC mdc;
  HBRUSH oBrush;
  HBITMAP obm;

  hdc = GetDC(NULL);                 
  mdc = CreateCompatibleDC(hdc);     // create display-compatible memory context
  hbm = CreateCompatibleBitmap(hdc, SCRWIDTH, SCRHEIGHT);
  obm = SelectObject(mdc, hbm);      // create DDB bitmap, select into context
                                     
  oBrush = SelectObject(hdc, CreateSolidBrush(bgcolor | 0x02000000L));  // brush to fill with 
  PatBlt(mdc, 0, 0, SCRWIDTH, SCRHEIGHT, PATCOPY);                      // CLEAR BITMAP
  DeleteObject(SelectObject(hdc, oBrush));

  draw_text_box(mdc, txt, fgcolor, margins, lspace, dotrack);  // DRAW THE TEXT

  SelectBitmap(mdc, obm);     // Release the GDI resources
  DeleteDC(mdc);
  ReleaseDC(NULL, hdc);
  return hbm;                 // Return the new bitmap
}

